import type { SavedChecklist, ChecklistItem } from '../types'

const STORAGE_KEY = 'gocheck-saved-checklists'

export function saveChecklist(checklist: SavedChecklist) {
    try {
      const existingChecklists = loadSavedChecklists()
      const existingIndex = existingChecklists.findIndex(c => c.uuid === checklist.uuid);

      if (existingIndex > -1) {
        // Update lastOpened timestamp if checklist already exists
        existingChecklists[existingIndex].lastOpened = new Date().toISOString();
        existingChecklists[existingIndex].name = checklist.name;
      } else {
        // Add new checklist with lastOpened timestamp
        existingChecklists.push({ ...checklist, lastOpened: new Date().toISOString() });
      }

      // Sort by lastOpened date in descending order
      existingChecklists.sort((a, b) => {
        if (!a.lastOpened) return 1;
        if (!b.lastOpened) return -1;
        return new Date(b.lastOpened).getTime() - new Date(a.lastOpened).getTime();
      });

      localStorage.setItem(STORAGE_KEY, JSON.stringify(existingChecklists));
    } catch (error) {
      console.error('Error saving checklist:', error);
    }
}

export const loadSavedChecklists = (): SavedChecklist[] => {
  const saved = localStorage.getItem(STORAGE_KEY)
  if (saved) {
  try {
    const parsed = JSON.parse(saved);
    if (Array.isArray(parsed)) {
      return parsed as SavedChecklist[];
    }
  } catch (error) {
    console.error('Error parsing saved checklists:', error);
  }
  }
  return []
}

export const exportChecklistToJSON = async (uuid: string, name: string): Promise<void> => {
  try {
    const response = await fetch(`/api/checklists/${uuid}/items`)
    if (!response.ok) {
      throw new Error('Failed to fetch checklist data')
    }
    
    const data = await response.json()
    if (!data.success || !data.items) {
      throw new Error('Invalid response format')
    }

    const exportData = {
      name: name,
      uuid: uuid,
      exportedAt: new Date().toISOString(),
      items: data.items as ChecklistItem[]
    }

    const jsonString = JSON.stringify(exportData, null, 2)
    const blob = new Blob([jsonString], { type: 'application/json' })
    const url = URL.createObjectURL(blob)
    
    const link = document.createElement('a')
    link.href = url
    link.download = `${name.replace(/[^a-z0-9]/gi, '_').toLowerCase()}_${uuid}.json`
    document.body.appendChild(link)
    link.click()
    document.body.removeChild(link)
    URL.revokeObjectURL(url)
  } catch (error) {
    console.error('Error exporting checklist:', error)
    throw error
  }
}

interface ImportData {
  name: string
  uuid?: string
  exportedAt?: string
  items: ChecklistItem[]
}

export const importChecklistFromJSON = async (file: File): Promise<string> => {
  try {
    // Read the file
    const text = await file.text()
    const importData: ImportData = JSON.parse(text)
    
    // Validate the import data
    if (!importData.name || !importData.items || !Array.isArray(importData.items)) {
      throw new Error('Invalid import file format')
    }

    // Create a new checklist
    const createResponse = await fetch('/api/checklists', {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
      },
      body: JSON.stringify({ name: importData.name }),
    })

    if (!createResponse.ok) {
      throw new Error('Failed to create checklist')
    }

    const createData = await createResponse.json()
    if (!createData.success || !createData.uuid) {
      throw new Error('Invalid response from checklist creation')
    }

    const newUuid = createData.uuid

    // Create a mapping from old item IDs to new item IDs
    const idMapping = new Map<number, number>()
    
    // First pass: Add all items and build the ID mapping
    for (const item of importData.items) {
      const addItemResponse = await fetch(`/api/checklists/${newUuid}/items`, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
        },
        body: JSON.stringify({
          content: item.content,
          parent_id: null, // We'll handle parent relationships in the second pass
          not_before: item.not_before,
          not_after: item.not_after,
        }),
      })

      if (!addItemResponse.ok) {
        throw new Error(`Failed to add item: ${item.content}`)
      }

      const addItemData = await addItemResponse.json()
      if (!addItemData.success || !addItemData.item) {
        throw new Error(`Invalid response when adding item: ${item.content}`)
      }

      const newItemId = addItemData.item.id
      idMapping.set(item.id, newItemId)
    }

    // Second pass: Update parent relationships and dependencies
    for (const item of importData.items) {
      const newItemId = idMapping.get(item.id)
      if (!newItemId) continue

      const updates: Partial<ChecklistItem> = {}

      // Handle parent relationships
      if (item.parent_id !== null) {
        const newParentId = idMapping.get(item.parent_id)
        if (newParentId) {
          updates.parent_id = newParentId
        }
      }

      // Handle dependencies
      if (item.dependencies && item.dependencies.length > 0) {
        const newDependencies = item.dependencies
          .map(oldDepId => idMapping.get(oldDepId))
          .filter(newDepId => newDepId !== undefined) as number[]
        
        if (newDependencies.length > 0) {
          updates.dependencies = newDependencies
        }
      }

      // Update item if it was checked in the original
      if (item.checked) {
        updates.checked = true
      }

      // Apply updates if there are any
      if (Object.keys(updates).length > 0) {
        const updateResponse = await fetch(`/api/checklists/${newUuid}/items/${newItemId}`, {
          method: 'PATCH',
          headers: {
            'Content-Type': 'application/json',
          },
          body: JSON.stringify(updates),
        })

        if (!updateResponse.ok) {
          console.warn(`Failed to update item: ${item.content}`)
        }
      }
    }

    return newUuid
  } catch (error) {
    console.error('Error importing checklist:', error)
    throw error
  }
}
